{----------------------------------------------------}
{ Andreis LCARS-Components 1.0 for Delphi (2, 3), 4 }
{ Copyright (c) 1999 Andrei Scheibner                }
{ http://www.htwm.de/~ascheibn                       }
{ ascheibn@htwm.de                                   }
{----------------------------------------------------}
{These Components are Freeware, you can distribute it,}
{you can improve it (because these are my first Delphi}
{components this may be necessary), but I would appreciate}
{it, if my name appears somewhere in your improvements}

unit LCARSRadioButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TCaptionPosition = (cpCenter, cpTopLeft, cpTopRight, cpBottomRight, cpBottomLeft);
  TLCARSRadioButton = class(TGraphicControl)
  private
   FCheckedColor : TColor;
   FUncheckedColor : TColor;
   FDisabledColor : TColor;
   FChecked : boolean;
   FBack : TBitmap;
   FBackColor : TColor;
   FCaption : TCaption;
   FCaptionWidth : integer;
   FCaptionHeight : integer;
   FCaptionPosX : integer;
   FCaptionPosY : integer;
   FRadius : integer;
   FCaptionMarginX : integer;
   FCaptionMarginY : integer;
   FCaptionPosition : TCaptionPosition;
   FAlignment: TLeftRight;
   FAllowGrayed: Boolean;
   FHorSpace : integer;
   FRectWidth : integer;
   procedure SetUncheckedColor(value : TColor);
   procedure SetCheckedColor(value : TColor);
   procedure SetDisabledColor(value : TColor);
   procedure SetCaption(value : TCaption);
   procedure SetCaptionPosition(value : TCaptionPosition);
   procedure SetCaptionMarginX(value : integer);
   procedure SetCaptionMarginY(value : integer);
   procedure SetAlignment(Value: TLeftRight);
   procedure SetChecked(Value: Boolean);
   procedure CMEnabledChanged(var Msg: TMessage); message CM_ENABLEDCHANGED;
  protected
   procedure Paint; override;
   procedure Click; override;
  public
   constructor Create(AOwner : TComponent) ; override;
   destructor Destroy; override;
  published
   property Parent;
   property Width default 129;
   property Height default 25;
   property Visible default true;
   property Enabled default true;
   property Font;
   property ParentFont default true;
   property Alignment: TLeftRight read fAlignment write SetAlignment default taRightJustify;
   property AllowGrayed: Boolean read fAllowGrayed write fAllowGrayed default False;
   property CaptionPosition : TCaptionPosition read FCaptionPosition
    write SetCaptionPosition default cpCenter;
   property Caption : TCaption read FCaption
    write SetCaption;
   property UncheckedColor : TColor read FUncheckedColor
    write SetUncheckedColor default clolive;
   property CheckedColor : TColor read FCheckedColor
    write SetCheckedColor default clyellow;
   property DisabledColor : TColor read FDisabledColor
    write SetDisabledColor default clgray;
   property CaptionMarginX : integer read FCaptionMarginX
    write SetCaptionMarginX default 2;
   property CaptionMarginY : integer read FCaptionMarginY
    write SetCaptionMarginY default 0;
   property Checked : boolean read FChecked
    write SetChecked default false;
   property OnClick;
   property OnDblClick;
   property OnMouseDown;
   property OnMouseUp;
   property OnMouseMove;
   property OnDragDrop;
   property OnDragOver;
   property OnEndDrag;
  end;

procedure Register;

implementation

constructor TLCARSRadioButton.Create(AOwner : TComponent);
begin
 inherited Create(AOwner);
 Width := 129;
 Height := 25;
 UncheckedColor := clolive;
 CheckedColor := clyellow;
 DisabledColor := clgray;
 Visible := true;
 Alignment := taRightJustify;
 AllowGrayed := false;
 CaptionPosition := cpCenter;
 FRadius := Height div 2;
 FHorSpace := 8;
 FBack := TBitmap.Create;
 CaptionMarginX := 2;
 CaptionMarginY := 0;
 ParentFont := true;
end;

destructor TLCARSRadioButton.Destroy;
begin
 FBack.Free;
 inherited Destroy;
end;

procedure TLCARSRadioButton.Click;
begin
 SetChecked(true);
 inherited Click;
end;

procedure TLCARSRadioButton.Paint;
var VisibleRect, BackRect : TRect;
begin
 FBack.Canvas.Font.Assign(Font);
 FBack.Width := Width;
 FBack.Height := Height;
 if Parent <> nil then
  with Parent do
   FBackColor := Color;
 FRadius := Height div 2;
 FRectWidth := Width - FRadius - FHorSpace;
 FCaptionWidth := FBack.Canvas.TextWidth(FCaption);
 FCaptionHeight := FBack.Canvas.TextHeight(FCaption);
 FBack.Canvas.Brush.Color := FBackColor;
 FBack.Canvas.Pen.Color := FBackColor;
 FBack.Canvas.Rectangle(0, 0, FBack.Width, FBack.Height);
 if Enabled then
  begin
   if Checked then
    begin
     FBack.Canvas.Brush.Color := CheckedColor;
     FBack.Canvas.Pen.Color := CheckedColor;
    end
   else if not Checked then
    begin
     FBack.Canvas.Brush.Color := UncheckedColor;
     FBack.Canvas.Pen.Color := UncheckedColor;
    end;
  end
 else
  begin
   FBack.Canvas.Brush.Color := DisabledColor;
   FBack.Canvas.Pen.Color := DisabledColor;
  end;
 if Alignment = taRightJustify then
  begin
   FBack.Canvas.MoveTo(FRadius, 0);
   FBack.Canvas.LineTo(FRadius, FBack.Height);
   FBack.Canvas.Arc(0, 0, FBack.Height, FBack.Height,
                    FRadius, 0, FRadius, FBack.Height);
   FBack.Canvas.FloodFill(FRadius - 2, FRadius,
                          FBack.Canvas.Pixels[FRadius - 2,
                          FRadius], fssurface);

  FBack.Canvas.Rectangle(FRadius, 0,
                          FRadius + 4, FBack.Height);
   FBack.Canvas.FloodFill(FRadius + 2, FRadius,
                          FBack.Canvas.Pixels[FRadius + 2, FRadius],
                          fssurface);

   FBack.Canvas.Rectangle(FRadius + FHorSpace, 0,
                          FBack.Width, FBack.Height);
   FBack.Canvas.FloodFill(FBack.width - 2, FRadius,
                          FBack.Canvas.Pixels[FBack.width - 2, FRadius],
                          fssurface);
  end
 else if Alignment = taLeftJustify then
  begin
   FBack.Canvas.MoveTo(FBack.Width - FRadius, 0);
   FBack.Canvas.LineTo(FBack.Width - FRadius, FBack.Height);
   FBack.Canvas.Arc(FBack.Width - FBack.Height, 0, FBack.Width ,FBack.Height,
                    FBack.Width - FRadius, FBack.Height,
                    FBack.Width - FRadius, 0);
   FBack.Canvas.FloodFill(FBack.Width - 2, FRadius,
                          FBack.Canvas.Pixels[FBack.Width - 2,
                          FRadius], fssurface);

   FBack.Canvas.Rectangle(FBack.Width - FRadius - 4, 0,
                          FBack.Width - FRadius, FBack.Height);
   FBack.Canvas.FloodFill(FBack.Width - FRadius - 2, FRadius,
                          FBack.Canvas.Pixels[FBack.Width - FRadius - 2, FRadius],
                          fssurface);

   FBack.Canvas.Rectangle(0, 0, FRectWidth, FBack.Height);
   FBack.Canvas.FloodFill(2, FRadius, FBack.Canvas.Pixels[2, FRadius],
                          fssurface);
  end;
 case FCaptionPosition of
  cpCenter :
   begin
    if Alignment = taRightJustify then
     begin
      FCaptionPosX := FRadius + FHorSpace + (FRectWidth div 2) - (FCaptionWidth div 2);
      FCaptionPosY := (FBack.Height div 2) - (FCaptionHeight div 2);
     end
    else if Alignment = taLeftJustify then
     begin
      FCaptionPosX := (FRectWidth div 2) - (FCaptionWidth div 2);
      FCaptionPosY := (FBack.Height div 2) - (FCaptionHeight div 2);
     end;
   end;
  cpTopLeft :
   begin
    if Alignment = taRightJustify then
     begin
      FCaptionPosX := FRadius + FHorSpace + CaptionMarginX;
      FCaptionPosY := CaptionMarginY;
     end
    else if Alignment = taLeftJustify then
     begin
      FCaptionPosX := CaptionMarginX;
      FCaptionPosY := CaptionMarginY;
     end;
   end;
  cpTopRight :
   begin
    if Alignment = taRightJustify then
     begin
      FCaptionPosX := FBack.Width - CaptionMarginX - FCaptionWidth;
      FCaptionPosY := CaptionMarginY;
     end
    else if Alignment = taLeftJustify then
     begin
      FCaptionPosX := FRectWidth - CaptionMarginX - FCaptionWidth;
      FCaptionPosY := CaptionMarginY;
     end;
   end;
  cpBottomRight :
   begin
    if Alignment = taRightJustify then
     begin
      FCaptionPosX := FBack.Width - CaptionMarginX - FCaptionWidth;
      FCaptionPosY := FBack.Height - CaptionMarginY - FCaptionHeight;
     end
    else if Alignment = taLeftJustify then
     begin
      FCaptionPosX := FRectWidth - CaptionMarginX - FCaptionWidth;
      FCaptionPosY := FBack.Height - CaptionMarginY - FCaptionHeight;
     end;
   end;
  cpBottomLeft :
   begin
    if Alignment = taRightJustify then
     begin
      FCaptionPosX := FRAdius + FHorSpace + CaptionMarginX;
      FCaptionPosY := FBack.Height - CaptionMarginY - FCaptionHeight;
     end
    else if Alignment = taLeftJustify then
     begin
      FCaptionPosX := CaptionMarginX;
      FCaptionPosY := FBack.Height - CaptionMarginY - FCaptionHeight;
     end;
   end;
 end;
 FBack.Canvas.TextOut(FCaptionPosX, FCaptionPosY, FCaption);
 VisibleRect := Rect(0, 0, Width, Height);
 BackRect := Rect(0, 0, FBack.Width, Fback.Height);
 Canvas.CopyRect(VisibleRect, FBack.Canvas, BackRect);
end;

procedure TLCARSRadioButton.SetAlignment(Value: TLeftRight);
begin
 if Value <> FAlignment then
  begin
   FAlignment := Value;
   Invalidate;
  end;
end;

procedure TLCARSRadioButton.SetChecked(Value: Boolean);

procedure TurnSiblingsOff;
var I: Integer;
    Sibling: TControl;
begin
 if Parent <> nil then
  with Parent do
   for I := 0 to ControlCount - 1 do
    begin
     Sibling := Controls[I];
     if (Sibling <> Self) and (Sibling is TLCARSRadioButton) then
      TLCARSRadioButton(Sibling).SetChecked(False);
    end;
end;

begin
  if FChecked <> Value then
   begin
    FChecked := Value;
    if Value then
     TurnSiblingsOff;
   end;
  Invalidate;
end;

procedure TLCARSRadioButton.CMEnabledChanged(var Msg: TMessage);
begin
 inherited;
 Invalidate;
end;

procedure TLCARSRadioButton.SetCaption(value : TCaption);
begin
 if FCaption <> value then
  begin
   FCaption := value;
   Invalidate;
  end;
end;

procedure TLCARSRadioButton.SetCaptionPosition(value : TCaptionPosition);
begin
 if FCaptionPosition <> value then
  begin
   FCaptionPosition := value;
   Invalidate;
  end;
end;

procedure TLCARSRadioButton.SetUncheckedColor(value : TColor);
begin
 if FUncheckedColor <> value then
  begin
   FUncheckedColor := value;
   Invalidate;
  end;
end;

procedure TLCARSRadioButton.SetCheckedColor(value : TColor);
begin
 if FCheckedColor <> value then
  begin
   FCheckedColor := value;
   Invalidate;
  end;
end;

procedure TLCARSRadioButton.SetDisabledColor(value : TColor);
begin
 if FDisabledColor <> value then
  begin
   FDisabledColor := value;
   Invalidate;
  end;
end;

procedure TLCARSRadioButton.SetCaptionMarginX(value : integer);
begin
 if FCaptionMarginX <> value then
  begin
   FCaptionMarginX := value;
   Invalidate;
  end;
end;

procedure TLCARSRadioButton.SetCaptionMarginY(value : integer);
begin
 if FCaptionMarginY <> value then
  begin
   FCaptionMarginY := value;
   Invalidate;
  end;
end;

procedure Register;
begin
  RegisterComponents('LCARS-Comps', [TLCARSRadioButton]);
end;

end.
